<?php
/**
 * The own template engine for loading caching and sending out images
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ImageTemplateEngine extends BaseTemplateEngine implements CompileableTemplate {
	/**
	 * Main nodes in the XML tree ('image' is ignored)
	 */
	private $mainNodes = array("base", "type", "resolution", "background-color", "foreground-color", "image-string");

	/**
	 * Sub nodes in the XML tree
	 */
	private $subNodes = array("name", "string-name", "x", "y", "font-size", "width", "height", "red", "green", "blue", "text");

	/**
	 * Image instance
	 */
	private $imageInstance = null;

	/**
	 * Current main node
	 */
	private $currMainNode = "";

	/**
	 * Protected constructor
	 *
	 * @return	void
	 */
	protected function __construct () {
		// Call parent constructor
		parent::__construct(__CLASS__);
	}

	/**
	 * Creates an instance of the class TemplateEngine and prepares it for usage
	 *
	 * @param	$basePath		The local base path for all templates
	 * @param	$langInstance	An instance of LanguageSystem (default)
	 * @param	$ioInstance		An instance of FileIoHandler (default, middleware!)
	 * @return	$tplInstance	An instance of TemplateEngine
	 * @throws	BasePathIsEmptyException		If the provided $basePath is empty
	 * @throws	InvalidBasePathStringException	If $basePath is no string
	 * @throws	BasePathIsNoDirectoryException	If $basePath is no
	 *											directory or not found
	 * @throws	BasePathReadProtectedException	If $basePath is
	 *											read-protected
	 */
	public final static function createImageTemplateEngine ($basePath, ManageableLanguage  $langInstance, FileIoHandler $ioInstance) {
		// Get a new instance
		$tplInstance = new ImageTemplateEngine();

		// Is the base path valid?
		if (empty($basePath)) {
			// Base path is empty
			throw new BasePathIsEmptyException($tplInstance, self::EXCEPTION_UNEXPECTED_EMPTY_STRING);
		} elseif (!is_string($basePath)) {
			// Is not a string
			throw new InvalidBasePathStringException(array($tplInstance, $basePath), self::EXCEPTION_INVALID_STRING);
		} elseif (!is_dir($basePath)) {
			// Is not a path
			throw new BasePathIsNoDirectoryException(array($tplInstance, $basePath), self::EXCEPTION_INVALID_PATH_NAME);
		} elseif (!is_readable($basePath)) {
			// Is not readable
			throw new BasePathReadProtectedException(array($tplInstance, $basePath), self::EXCEPTION_READ_PROTECED_PATH);
		}

		// Get configuration instance
		$cfgInstance = FrameworkConfiguration::getInstance();

		// Set the base path
		$tplInstance->setBasePath($basePath);

		// Set the language and IO instances
		$tplInstance->setLanguageInstance($langInstance);
		$tplInstance->setFileIoInstance($ioInstance);

		// Set template extensions
		$tplInstance->setRawTemplateExtension($cfgInstance->readConfig('raw_template_extension'));
		$tplInstance->setCodeTemplateExtension($cfgInstance->readConfig('code_template_extension'));

		// Absolute output path for compiled templates
		$tplInstance->setCompileOutputPath($cfgInstance->readConfig('base_path') . $cfgInstance->readConfig('compile_output_path'));

		// Return the prepared instance
		return $tplInstance;
	}

	/**
	 * Getter for current main node
	 *
	 * @return	$currMainNode	Current main node
	 */
	public final function getCurrMainNode () {
		return $this->currMainNode;
	}

	/**
	 * Getter for main node array
	 *
	 * @return	$mainNodes	Array with valid main node names
	 */
	public final function getMainNodes () {
		return $this->mainNodes;
	}

	/**
	 * Getter for sub node array
	 *
	 * @return	$subNodes	Array with valid sub node names
	 */
	public final function getSubNodes () {
		return $this->subNodes;
	}

	/**
	 * Handles the start element of an XML resource
	 *
	 * @param	$resource		XML parser resource (currently ignored)
	 * @param	$element		The element we shall handle
	 * @param	$attributes		All attributes
	 * @return	void
	 * @throws	InvalidXmlNodeException		If an unknown/invalid XML node name was found
	 */
	public function startElement ($resource, $element, array $attributes) {
		// Initial method name which will never be called...
		$methodName = 'initImage';

		// Make the element name lower-case
		$element = strtolower($element);

		// Is the element a main node?
		//* DEBUG: */ echo "START: &gt;".$element."&lt;<br />\n";
		if (in_array($element, $this->mainNodes)) {
			// Okay, main node found!
			$methodName = 'setImage' . $this->convertToClassName($element);
		} elseif (in_array($element, $this->subNodes)) {
			// Sub node found
			$methodName = 'setImageProperty' . $this->convertToClassName($element);
		} elseif ($element != 'image') {
			// Invalid node name found
			throw new InvalidXmlNodeException(array($this, $element, $attributes), BaseHelper::EXCEPTION_XML_NODE_UNKNOWN);
		}

		// Call method
		//* DEBUG: */ echo "call: ".$methodName."<br />\n";
		call_user_func_array(array($this, $methodName), $attributes);
	}

	/**
	 * Ends the main or sub node by sending out the gathered data
	 *
	 * @param	$resource	An XML resource pointer (currently ignored)
	 * @param	$nodeName	Name of the node we want to finish
	 * @return	void
	 * @throws	XmlNodeMismatchException	If current main node mismatches the closing one
	 */
	protected function endElement ($resource, $nodeName) {
		// Make all lower-case
		$nodeName = strtolower($nodeName);

		// Does this match with current main node?
		//* DEBUG: */ echo "END: &gt;".$nodeName."&lt;<br />\n";
		if (($nodeName != $this->getCurrMainNode()) && (in_array($nodeName, $this->getMainNodes()))) {
			// Did not match!
			throw new XmlNodeMismatchException (array($this, $nodeName, $this->getCurrMainNode()), BaseHelper::EXCEPTION_XML_NODE_MISMATCH);
		} elseif (in_array($nodeName, $this->getSubNodes())) {
			// Silently ignore sub nodes
			return;
		}

		// Construct method name
		$methodName = 'finish' . $this->convertToClassName($nodeName);

		// Call the corresponding method
		call_user_func_array(array($this->imageInstance, $methodName), array());
	}

	/**
	 * Currently not used
	 *
	 * @param	$resource		XML parser resource (currently ignored)
	 * @param	$characters		Characters to handle
	 * @return	void
	 * @todo	Find something usefull with this!
	 */
	protected function characterHandler ($resource, $characters) {
		// Trim all spaces away
		$characters = trim($characters);

		// Is this string empty?
		if (empty($characters)) {
			// Then skip it silently
			return false;
		} // END - if

		// Unfinished work!
		$this->partialStub("Handling extra characters is not yet supported!");
	}

	/**
	 * Intializes the image
	 *
	 * @return	void
	 * @todo	Add cache creation here
	 */
	private function initImage () {
		// Unfinished work!
	}

	/**
	 * Set the image type
	 *
	 * @param	$imageType	Code fragment or direct value holding the image type
	 * @return	void
	 */
	private function setImageType ($imageType) {
		// Set group to general
		$this->setVariableGroup('general');

		// Try to compile it first to get the value from variable stack
		$imageType = $this->compileRawCode($imageType);

		// Now make a class name of it
		$className = $this->convertToClassName($imageType.'_image');

		// And try to initiate it
		$this->imageInstance = ObjectFactory::createObjectByName($className, array($this));

		// Set current main node to type
		$this->currMainNode = 'type';
	}

	/**
	 * "Setter" for resolution, we first need to collect the resolution from the
	 * sub-nodes. So first, this method will prepare an array for it
	 *
	 * @return	void
	 */
	private function setImageResolution () {
		// Call the image class
		$this->imageInstance->initResolution();

		// Current main node is resolution
		$this->currMainNode = 'resolution';
	}

	/**
	 * "Setter" for base information. For more details see above method!
	 *
	 * @return	void
	 * @see		ImageTemplateEngine::setImageResolution
	 */
	private function setImageBase () {
		// Call the image class
		$this->imageInstance->initBase();

		// Current main node is resolution
		$this->currMainNode = 'base';
	}

	/**
	 * "Setter" for background-color. For more details see above method!
	 *
	 * @return	void
	 * @see		ImageTemplateEngine::setImageResolution
	 */
	private function setImageBackgroundColor () {
		// Call the image class
		$this->imageInstance->initBackgroundColor();

		// Current main node is background-color
		$this->currMainNode = 'background-color';
	}

	/**
	 * "Setter" for foreground-color. For more details see above method!
	 *
	 * @return	void
	 * @see		ImageTemplateEngine::setImageResolution
	 */
	private function setImageForegroundColor () {
		// Call the image class
		$this->imageInstance->initForegroundColor();

		// Current main node is foreground-color
		$this->currMainNode = 'foreground-color';
	}

	/**
	 * "Setter" for image-string. For more details see above method!
	 *
	 * @param	$groupable	Wether this image string is groupable
	 * @return	void
	 * @see		ImageTemplateEngine::setImageResolution
	 */
	private function setImageImageString ($groupable = 'single') {
		// Call the image class
		$this->imageInstance->initImageString($groupable);

		// Current main node is foreground-color
		$this->currMainNode = 'image-string';
	}

	/**
	 * Setter for image name
	 *
	 * @param	$imageName	Name of the image
	 * @return	void
	 */
	private function setImagePropertyName ($imageName) {
		// Call the image class
		$this->imageInstance->setImageName($imageName);
	}

	/**
	 * Setter for image width
	 *
	 * @param	$width	Width of the image or variable
	 * @return	void
	 */
	private function setImagePropertyWidth ($width) {
		// Call the image class
		$this->imageInstance->setWidth($width);
	}

	/**
	 * Setter for image height
	 *
	 * @param	$height	Height of the image or variable
	 * @return	void
	 */
	private function setImagePropertyHeight ($height) {
		// Call the image class
		$this->imageInstance->setHeight($height);
	}

	/**
	 * Setter for image red color
	 *
	 * @param	$red	Red color value
	 * @return	void
	 */
	private function setImagePropertyRed ($red) {
		// Call the image class
		$this->imageInstance->setRed($red);
	}

	/**
	 * Setter for image green color
	 *
	 * @param	$green	Green color value
	 * @return	void
	 */
	private function setImagePropertyGreen ($green) {
		// Call the image class
		$this->imageInstance->setGreen($green);
	}

	/**
	 * Setter for image blue color
	 *
	 * @param	$blue	Blue color value
	 * @return	void
	 */
	private function setImagePropertyBlue ($blue) {
		// Call the image class
		$this->imageInstance->setBlue($blue);
	}

	/**
	 * Setter for string name (identifier)
	 *
	 * @param	$stringName		String name (identifier)
	 * @return	void
	 */
	private function setImagePropertyStringName ($stringName) {
		// Call the image class
		$this->imageInstance->setStringName($stringName);
	}

	/**
	 * Setter for font size
	 *
	 * @param	$fontSize	Size of the font
	 * @return	void
	 */
	private function setImagePropertyFontSize ($fontSize) {
		// Call the image class
		$this->imageInstance->setFontSize($fontSize);
	}

	/**
	 * Setter for image string
	 *
	 * @param	$imageString	Image string to set
	 * @return	void
	 */
	private function setImagePropertyText ($imageString) {
		// Call the image class
		$this->imageInstance->setString($imageString);
	}

	/**
	 * Setter for X coordinate
	 *
	 * @param	$x	X coordinate
	 * @return	void
	 */
	private function setImagePropertyX ($x) {
		// Call the image class
		$this->imageInstance->setX($x);
	}

	/**
	 * Setter for Y coordinate
	 *
	 * @param	$y	Y coordinate
	 * @return	void
	 */
	private function setImagePropertyY ($y) {
		// Call the image class
		$this->imageInstance->setY($y);
	}

	/**
	 * Getter for image cache file (FQFN)
	 *
	 * @return	$fqfn	Full-qualified file name of the image cache
	 */
	public function getImageCacheFqfn () {
		// Get the FQFN ready
		$fqfn = $this->getBasePath().'_cache/' . md5($this->imageInstance->getImageName().":".$this->__toString().":".$this->imageInstance->__toString()) . "." . $this->imageInstance->getImageType();

		// Return it
		return $fqfn;
	}

	/**
	 * Outputs the image to the world
	 *
	 * @param	$responseInstance	An instance of a Responseable class
	 * @return	void
	 */
	public function transferToResponse (Responseable $responseInstance) {
		// Set the image instance
		$responseInstance->setImageInstance($this->imageInstance);
	}
}

// [EOF]
?>
